/*
 *	gtk_common.h
 *	
 *	(C) 2001 Robert Kling
 *	robkli-8@student.luth.se
 *	http://boombox.campus.luth.se
 *	
 *	Based on code (C) 2000 Daniel Sundberg
 *	
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <gtk/gtk.h>

#ifndef __GTK_COMMON_H
#define __GTK_COMMON_H

GtkWidget *xpm_label_box(GtkWidget *parent,  char *xpm_data[], gchar *label_text);
gchar *xpm_label_box_get_text(GtkWidget *);
GtkWidget *gtk_button_new_with_label_with_pixmap(gchar *label,  char *xpm_data[]);
gchar *gtk_button_get_text(GtkWidget *button);
GtkWidget *gtk_menu_item_new_with_label_with_pixmap(gchar *label,  char *xpm_data[]);
gchar *gtk_menu_item_get_text(GtkWidget *menu_item);
void popup_dialog(int height, gboolean wrap, char *title, char *text, char *button_text);
gint close_popup_dialog(GtkWidget *widget, gpointer data);

#endif
